@@ -456,7 +456,7 @@ defmodule Time do
   @doc """
   Adds the `amount_to_add` of `unit`s to the given `time`.
 
-  Accepts an `amount_to_add` in any `unit`. `unit` can be `:day`,
+  Accepts an `amount_to_add` in any `unit`. `unit` can be
   `:hour`, `:minute`, `:second` or any subsecond precision from
   `t:System.time_unit/0`. It defaults to `:second`. Negative values
   will move backwards in time.
@@ -598,6 +598,44 @@ defmodule Time do
     end
   end
 
+  @doc """
+  Returns `true` if the first time is strictly earlier than the second.
+
+  ## Examples
+
+      iex> Time.before?(~T[16:04:16], ~T[16:04:28])
+      true
+      iex> Time.before?(~T[16:04:16], ~T[16:04:16])
+      false
+      iex> Time.before?(~T[16:04:16.01], ~T[16:04:16.001])
+      false
+
+  """
+  @doc since: "1.15.0"
+  @spec before?(Calendar.time(), Calendar.time()) :: boolean()
+  def before?(time1, time2) do
+    compare(time1, time2) == :lt
+  end
+
+  @doc """
+  Returns `true` if the first time is strictly later than the second.
+
+  ## Examples
+
+      iex> Time.after?(~T[16:04:28], ~T[16:04:16])
+      true
+      iex> Time.after?(~T[16:04:16], ~T[16:04:16])
+      false
+      iex> Time.after?(~T[16:04:16.001], ~T[16:04:16.01])
+      false
+
+  """
+  @doc since: "1.15.0"
+  @spec after?(Calendar.time(), Calendar.time()) :: boolean()
+  def after?(time1, time2) do
+    compare(time1, time2) == :gt
+  end
+
   @doc """
   Converts given `time` to a different calendar.
 
