@@ -64,6 +64,12 @@ defmodule System do
   """
 
   defmodule EnvError do
+    @moduledoc """
+    An exception raised when a system environment variable is not set.
+
+    For example, see `System.fetch_env!/1`.
+    """
+
     defexception [:env]
 
     @impl true
@@ -749,9 +755,12 @@ defmodule System do
   Sets multiple environment variables.
 
   Sets a new value for each environment variable corresponding
-  to each `{key, value}` pair in `enum`. Keys are automatically
-  converted to strings, values are sent as is. `nil` values erase
+  to each `{key, value}` pair in `enum`. Keys and non-nil values
+  are automatically converted to charlists. `nil` values erase
   the given keys.
+
+  Overall, this is a convenience wrapper around `put_env/2` and
+  `delete_env/2` with support for different key and value formats.
   """
   @spec put_env(Enumerable.t()) :: :ok
   def put_env(enum) do
@@ -996,25 +1005,6 @@ defmodule System do
   `Port` module describes this problem and possible solutions under
   the "Zombie processes" section.
 
-  > #### Windows argument splitting and untrusted arguments {: .warning}
-  >
-  > On Unix systems, arguments are passed to a new operating system
-  > process as an array of strings but on Windows it is up to the child
-  > process to parse them and some Windows programs may apply their own
-  > rules, which are inconsistent with the standard C runtime `argv` parsing
-  >
-  > This is particularly troublesome when invoking `.bat` or `.com` files
-  > as these run implicitly through `cmd.exe`, whose argument parsing is
-  > vulnerable to malicious input and can be used to run arbitrary shell
-  > commands.
-  >
-  > Therefore, if you are running on Windows and you execute batch
-  > files or `.com` applications, you must not pass untrusted input as
-  > arguments to the program. You may avoid accidentally executing them
-  > by explicitly passing the extension of the program you want to run,
-  > such as `.exe`, and double check the program is indeed not a batch
-  > file or `.com` application.
-
   ## Examples
 
       iex> System.cmd("echo", ["hello"])
@@ -1059,8 +1049,8 @@ defmodule System do
     * `:parallelism` - when `true`, the VM will schedule port tasks to improve
       parallelism in the system. If set to `false`, the VM will try to perform
       commands immediately, improving latency at the expense of parallelism.
-      The default is `false`, and can be set on system startup by passing the 
-      [`+spp`](https://www.erlang.org/doc/man/erl.html#+spp) flag to `--erl`. 
+      The default is `false`, and can be set on system startup by passing the
+      [`+spp`](https://www.erlang.org/doc/man/erl.html#+spp) flag to `--erl`.
       Use `:erlang.system_info(:port_parallelism)` to check if enabled.
 
   ## Error reasons
