@@ -91,6 +91,22 @@ defmodule CodeTest do
                end)
     end
 
+    test "includes column information on unknown remote function calls" do
+      sample = """
+      defmodule CodeTest.UnknownRemoteCall do
+        def perform do
+          UnkownModule.foo()
+        end
+      end
+      """
+
+      assert {_, [%{position: {3, 18}}]} =
+               Code.with_diagnostics(fn ->
+                 quoted = Code.string_to_quoted!(sample, columns: true)
+                 Code.eval_quoted(quoted, [])
+               end)
+    end
+
     test "captures unknown local calls" do
       sample = """
       defmodule CodeTest.UnknownLocalCall do
@@ -236,6 +252,7 @@ defmodule CodeTest do
                  message: "undefined variable \"x\"",
                  position: 1,
                  file: "nofile",
+                 source: "nofile",
                  stacktrace: [],
                  severity: :error
                }
@@ -322,6 +339,21 @@ defmodule CodeTest do
                {[{{:x, :foo}, 2}], [x: :foo]}
     end
 
+    if :erlang.system_info(:otp_release) >= ~c"25" do
+      test "undefined function" do
+        env = Code.env_for_eval(__ENV__)
+        quoted = quote do: foo()
+
+        assert_exception(
+          UndefinedFunctionError,
+          ["** (UndefinedFunctionError) function foo/0 is undefined (there is no such import)"],
+          fn ->
+            Code.eval_quoted_with_env(quoted, [], env)
+          end
+        )
+      end
+    end
+
     defmodule Tracer do
       def trace(event, env) do
         send(self(), {:trace, event, env})
@@ -426,29 +458,37 @@ defmodule CodeTest do
   end
 
   test "string_to_quoted!/2 errors take lines and columns into account" do
-    message = "nofile:1:5: syntax error before: '*'\n    |\n  1 | 1 + * 3\n    |     ^"
+    assert_exception(
+      SyntaxError,
+      ["nofile:1:5:", "syntax error before:", "1 + * 3", "^"],
+      fn ->
+        Code.string_to_quoted!("1 + * 3")
+      end
+    )
 
-    assert_raise SyntaxError, message, fn ->
-      Code.string_to_quoted!("1 + * 3")
-    end
+    assert_exception(
+      SyntaxError,
+      ["nofile:10:5:", "syntax error before:", "1 + * 3", "^"],
+      fn ->
+        Code.string_to_quoted!("1 + * 3", line: 10)
+      end
+    )
 
-    message = "nofile:10:5: syntax error before: '*'\n    |\n 10 | 1 + * 3\n    |     ^"
+    assert_exception(
+      SyntaxError,
+      ["nofile:10:7:", "syntax error before:", "1 + * 3", "^"],
+      fn ->
+        Code.string_to_quoted!("1 + * 3", line: 10, column: 3)
+      end
+    )
 
-    assert_raise SyntaxError, message, fn ->
-      Code.string_to_quoted!("1 + * 3", line: 10)
-    end
-
-    message = "nofile:10:7: syntax error before: '*'\n    |\n 10 | 1 + * 3\n    |     ^"
-
-    assert_raise SyntaxError, message, fn ->
-      Code.string_to_quoted!("1 + * 3", line: 10, column: 3)
-    end
-
-    message = "nofile:11:5: syntax error before: '*'\n    |\n 11 | 1 + * 3\n    |     ^"
-
-    assert_raise SyntaxError, message, fn ->
-      Code.string_to_quoted!(":ok\n1 + * 3", line: 10, column: 3)
-    end
+    assert_exception(
+      SyntaxError,
+      ["nofile:11:7:", "syntax error before:", "1 + * 3", "^"],
+      fn ->
+        Code.string_to_quoted!(":ok\n1 + * 3", line: 10, column: 3)
+      end
+    )
   end
 
   test "string_to_quoted only requires the List.Chars protocol implementation to work" do
@@ -459,9 +499,13 @@ defmodule CodeTest do
   end
 
   test "string_to_quoted returns error on incomplete escaped string" do
-    assert Code.string_to_quoted("\"\\") ==
-             {:error,
-              {[line: 1, column: 3], "missing terminator: \" (for string starting at line 1)", ""}}
+    assert {:error, {meta, "missing terminator: \" (for string starting at line 1)", ""}} =
+             Code.string_to_quoted("\"\\")
+
+    assert meta[:line] == 1
+    assert meta[:column] == 1
+    assert meta[:end_line] == 1
+    assert meta[:end_column] == 3
   end
 
   test "compile source" do
@@ -589,6 +633,19 @@ defmodule CodeTest do
       assert Code.fetch_docs(Io) == {:error, :module_not_found}
     end
   end
+
+  defp assert_exception(ex, messages, callback) do
+    e =
+      assert_raise ex, fn ->
+        callback.()
+      end
+
+    error_msg = Exception.format(:error, e, [])
+
+    for msg <- messages do
+      assert error_msg =~ msg
+    end
+  end
 end
 
 defmodule Code.SyncTest do
