@@ -147,7 +147,7 @@ defmodule Kernel.ParallelCompilerTest do
       expected_msg = "Undef.__struct__/1 is undefined, cannot expand struct Undef"
 
       assert capture_io(:stderr, fn ->
-               assert {:error, [{^fixture, 3, msg}, {^fixture, 0, compile_msg}], []} =
+               assert {:error, [{^fixture, {3, 5}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.compile([fixture])
 
                assert msg =~ expected_msg
@@ -196,6 +196,28 @@ defmodule Kernel.ParallelCompilerTest do
       purge([QuickExample])
     end
 
+    test "does not crash on external reports" do
+      [fixture] =
+        write_tmp(
+          "compile_quoted",
+          quick_example: """
+          defmodule CompileQuoted do
+            try do
+              Code.compile_quoted({:fn, [], [{:->, [], [[], quote(do: unknown_var)]}]})
+            rescue
+              _ -> :ok
+            end
+          end
+          """
+        )
+
+      assert capture_io(:stderr, fn ->
+               assert {:ok, [CompileQuoted], []} = Kernel.ParallelCompiler.compile([fixture])
+             end) =~ "undefined variable \"unknown_var\""
+    after
+      purge([CompileQuoted])
+    end
+
     test "does not hang on missing dependencies" do
       [fixture] =
         write_tmp(
@@ -216,7 +238,7 @@ defmodule Kernel.ParallelCompilerTest do
         "ThisModuleWillNeverBeAvailable.__struct__/1 is undefined, cannot expand struct ThisModuleWillNeverBeAvailable"
 
       assert capture_io(:stderr, fn ->
-               assert {:error, [{^fixture, 7, msg}, {^fixture, 0, compile_msg}], []} =
+               assert {:error, [{^fixture, {7, 3}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.compile([fixture])
 
                assert msg =~ expected_msg
@@ -245,7 +267,9 @@ defmodule Kernel.ParallelCompilerTest do
         "ThisModuleWillNeverBeAvailable.__struct__/1 is undefined, cannot expand struct ThisModuleWillNeverBeAvailable"
 
       assert capture_io(:stderr, fn ->
-               assert {:error, [{^missing_struct, 2, msg}, {^missing_struct, 0, compile_msg}], []} =
+               assert {:error,
+                       [{^missing_struct, {2, 3}, msg}, {^missing_struct, 0, compile_msg}],
+                       []} =
                         Kernel.ParallelCompiler.compile([missing_struct, depends_on])
 
                assert msg =~ expected_msg
@@ -272,7 +296,9 @@ defmodule Kernel.ParallelCompilerTest do
       expected_msg = "module Unknown.Module is not loaded and could not be found"
 
       assert capture_io(:stderr, fn ->
-               assert {:error, [{^missing_import, 2, msg}, {^missing_import, 0, compile_msg}], []} =
+               assert {:error,
+                       [{^missing_import, {2, 3}, msg}, {^missing_import, 0, compile_msg}],
+                       []} =
                         Kernel.ParallelCompiler.compile([missing_import, depends_on])
 
                assert msg =~ expected_msg
@@ -301,7 +327,7 @@ defmodule Kernel.ParallelCompilerTest do
       msg =
         capture_io(:stderr, fn ->
           fixtures = [foo, bar]
-          assert {:error, [foo_error, bar_error], []} = Kernel.ParallelCompiler.compile(fixtures)
+          assert {:error, [bar_error, foo_error], []} = Kernel.ParallelCompiler.compile(fixtures)
           assert bar_error == {bar, nil, "deadlocked waiting on module FooDeadlock"}
           assert foo_error == {foo, nil, "deadlocked waiting on module BarDeadlock"}
         end)
@@ -337,7 +363,7 @@ defmodule Kernel.ParallelCompilerTest do
       purge([FooCircular, BarCircular])
     end
 
-    test "handles async compilation" do
+    test "handles pmap compilation" do
       [foo, bar] =
         write_tmp(
           "async_compile",
@@ -345,10 +371,11 @@ defmodule Kernel.ParallelCompilerTest do
           defmodule FooAsync do
             true = Code.can_await_module_compilation?()
 
-            Kernel.ParallelCompiler.async(fn ->
-              true = Code.can_await_module_compilation?()
-              BarAsync.__info__(:module)
-            end)
+            [BarAsync] =
+              Kernel.ParallelCompiler.pmap([:ok], fn :ok ->
+                true = Code.can_await_module_compilation?()
+                BarAsync.__info__(:module)
+              end)
           end
           """,
           bar: """
@@ -368,17 +395,15 @@ defmodule Kernel.ParallelCompilerTest do
       purge([FooAsync, BarAsync])
     end
 
-    test "handles async deadlocks" do
+    test "handles pmap deadlocks" do
       [foo, bar] =
         write_tmp(
           "async_deadlock",
           foo: """
           defmodule FooAsyncDeadlock do
-            Kernel.ParallelCompiler.async(fn ->
+            Kernel.ParallelCompiler.pmap([:ok], fn :ok ->
               BarAsyncDeadlock.__info__(:module)
             end)
-
-            BarAsyncDeadlock.__info__(:module)
           end
           """,
           bar: """
@@ -390,9 +415,9 @@ defmodule Kernel.ParallelCompilerTest do
 
       capture_io(:stderr, fn ->
         fixtures = [foo, bar]
-        assert {:error, [foo_error, bar_error], []} = Kernel.ParallelCompiler.compile(fixtures)
-        assert bar_error == {bar, nil, "deadlocked waiting on module FooAsyncDeadlock"}
-        assert foo_error == {foo, nil, "deadlocked waiting on module BarAsyncDeadlock"}
+        assert {:error, [bar_error, foo_error], []} = Kernel.ParallelCompiler.compile(fixtures)
+        assert {^bar, nil, "deadlocked waiting on module FooAsyncDeadlock"} = bar_error
+        assert {^foo, nil, "deadlocked waiting on pmap [#PID<" <> _} = foo_error
       end)
     end
 
@@ -477,6 +502,24 @@ defmodule Kernel.ParallelCompilerTest do
       end)
     end
 
+    test "gets correct file+line+column number for SyntaxError" do
+      File.mkdir_p!(tmp_path())
+
+      [fixture] =
+        write_tmp("error",
+          error: """
+          raise SyntaxError, file: "foo/bar.ex", line: 3, column: 10
+          """
+        )
+
+      file = Path.absname("foo/bar.ex")
+
+      capture_io(:stderr, fn ->
+        assert {:error, [%{file: ^file, source: ^fixture, position: {3, 10}}], _} =
+                 Kernel.ParallelCompiler.compile([fixture], return_diagnostics: true)
+      end)
+    end
+
     test "gets proper beam destinations from dynamic modules" do
       fixtures =
         write_tmp(
@@ -510,7 +553,7 @@ defmodule Kernel.ParallelCompilerTest do
       expected_msg = "Undef.__struct__/1 is undefined, cannot expand struct Undef"
 
       assert capture_io(:stderr, fn ->
-               assert {:error, [{^fixture, 3, msg}, {^fixture, 0, compile_msg}], []} =
+               assert {:error, [{^fixture, {3, 5}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.require([fixture])
 
                assert msg =~ expected_msg
@@ -538,7 +581,7 @@ defmodule Kernel.ParallelCompilerTest do
         "ThisModuleWillNeverBeAvailable.__struct__/1 is undefined, cannot expand struct ThisModuleWillNeverBeAvailable"
 
       assert capture_io(:stderr, fn ->
-               assert {:error, [{^fixture, 7, msg}, {^fixture, 0, compile_msg}], []} =
+               assert {:error, [{^fixture, {7, 3}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.require([fixture])
 
                assert msg =~ expected_msg
