@@ -86,24 +86,19 @@ defmodule KernelTest do
   test "=~/2" do
     assert "abcd" =~ ~r/c(d)/ == true
     assert "abcd" =~ ~r/e/ == false
-    assert "abcd" =~ ~R/c(d)/ == true
-    assert "abcd" =~ ~R/e/ == false
 
     string = "^ab+cd*$"
     assert string =~ "ab+" == true
     assert string =~ "bb" == false
 
     assert "abcd" =~ ~r// == true
-    assert "abcd" =~ ~R// == true
     assert "abcd" =~ "" == true
 
     assert "" =~ ~r// == true
-    assert "" =~ ~R// == true
     assert "" =~ "" == true
 
     assert "" =~ "abcd" == false
     assert "" =~ ~r/abcd/ == false
-    assert "" =~ ~R/abcd/ == false
 
     assert_raise FunctionClauseError, "no function clause matching in Kernel.=~/2", fn ->
       1234 =~ "hello"
@@ -114,7 +109,7 @@ defmodule KernelTest do
     end
 
     assert_raise FunctionClauseError, "no function clause matching in Kernel.=~/2", fn ->
-      1234 =~ ~R"hello"
+      1234 =~ ~r"hello"
     end
 
     assert_raise FunctionClauseError, "no function clause matching in Kernel.=~/2", fn ->
@@ -734,14 +729,6 @@ defmodule KernelTest do
       assert result =~ ":lists.member(rand(), [1 | some_call()]"
     end
 
-    defp quote_case_in(left, right) do
-      quote do
-        case 0 do
-          _ when unquote(left) in unquote(right) -> true
-        end
-      end
-    end
-
     test "is optimized" do
       assert expand_to_string(quote(do: foo in [])) ==
                "_ = foo\nfalse"
