@@ -10,13 +10,6 @@ defmodule Module.Types.UnifyTest do
     |> lift_result()
   end
 
-  defp unify_directed_lift(left, right) do
-    stack = %{new_stack() | context: :expr}
-
-    unify(left, right, stack, new_context())
-    |> lift_result()
-  end
-
   defp new_context() do
     Types.context("types_test.ex", TypesTest, {:test, 0}, [], Module.ParallelChecker.test_cache())
   end
@@ -34,13 +27,22 @@ defmodule Module.Types.UnifyTest do
   end
 
   defp lift_result({:ok, type, context}) do
-    {:ok, Types.lift_type(type, context)}
+    {:ok, lift_type(type, context)}
   end
 
   defp lift_result({:error, {type, reason, _context}}) do
     {:error, {type, reason}}
   end
 
+  defp lift_type(type, context) do
+    {[type], _context} = lift_types([type], context)
+    type
+  end
+
+  defp format_type_string(type, simplify?) do
+    IO.iodata_to_binary(format_type(type, simplify?))
+  end
+
   describe "unify/3" do
     test "literal" do
       assert unify_lift({:atom, :foo}, {:atom, :foo}) == {:ok, {:atom, :foo}}
@@ -57,14 +59,9 @@ defmodule Module.Types.UnifyTest do
       assert {:error, {:unable_unify, {:integer, :atom, _}}} = unify_lift(:integer, :atom)
     end
 
-    test "subtype undirected" do
-      assert unify_lift(:atom, {:atom, true}) == {:ok, {:atom, true}}
+    test "atom subtype" do
       assert unify_lift({:atom, true}, :atom) == {:ok, {:atom, true}}
-    end
-
-    test "subtype directed" do
-      assert unify_directed_lift({:atom, true}, :atom) == {:ok, {:atom, true}}
-      assert {:error, _} = unify_directed_lift(:atom, {:atom, true})
+      assert {:error, _} = unify_lift(:atom, {:atom, true})
     end
 
     test "tuple" do
@@ -123,12 +120,6 @@ defmodule Module.Types.UnifyTest do
              ) ==
                {:ok, {:map, [{:required, :integer, :atom}]}}
 
-      assert unify_lift(
-               {:map, [{:required, {:atom, :foo}, {:atom, :bar}}]},
-               {:map, [{:required, {:atom, :foo}, :atom}]}
-             ) ==
-               {:ok, {:map, [{:required, {:atom, :foo}, {:atom, :bar}}]}}
-
       assert {:error,
               {:unable_unify,
                {{:map, [{:required, :integer, :atom}]}, {:map, [{:required, :atom, :integer}]}, _}}} =
@@ -181,22 +172,22 @@ defmodule Module.Types.UnifyTest do
     end
 
     test "map with subtyped keys" do
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:required, {:atom, :foo}, :integer}]},
                {:map, [{:required, :atom, :integer}]}
              ) == {:ok, {:map, [{:required, {:atom, :foo}, :integer}]}}
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:optional, {:atom, :foo}, :integer}]},
                {:map, [{:required, :atom, :integer}]}
              ) == {:ok, {:map, [{:required, {:atom, :foo}, :integer}]}}
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:required, {:atom, :foo}, :integer}]},
                {:map, [{:optional, :atom, :integer}]}
              ) == {:ok, {:map, [{:required, {:atom, :foo}, :integer}]}}
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:optional, {:atom, :foo}, :integer}]},
                {:map, [{:optional, :atom, :integer}]}
              ) == {:ok, {:map, [{:optional, {:atom, :foo}, :integer}]}}
@@ -206,7 +197,7 @@ defmodule Module.Types.UnifyTest do
                {{:map, [{:required, :atom, :integer}]},
                 {:map, [{:required, {:atom, :foo}, :integer}]},
                 _}}} =
-               unify_directed_lift(
+               unify_lift(
                  {:map, [{:required, :atom, :integer}]},
                  {:map, [{:required, {:atom, :foo}, :integer}]}
                )
@@ -216,7 +207,7 @@ defmodule Module.Types.UnifyTest do
                {{:map, [{:optional, :atom, :integer}]},
                 {:map, [{:required, {:atom, :foo}, :integer}]},
                 _}}} =
-               unify_directed_lift(
+               unify_lift(
                  {:map, [{:optional, :atom, :integer}]},
                  {:map, [{:required, {:atom, :foo}, :integer}]}
                )
@@ -226,37 +217,170 @@ defmodule Module.Types.UnifyTest do
                {{:map, [{:required, :atom, :integer}]},
                 {:map, [{:optional, {:atom, :foo}, :integer}]},
                 _}}} =
-               unify_directed_lift(
+               unify_lift(
                  {:map, [{:required, :atom, :integer}]},
                  {:map, [{:optional, {:atom, :foo}, :integer}]}
                )
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:optional, :atom, :integer}]},
                {:map, [{:optional, {:atom, :foo}, :integer}]}
              ) == {:ok, {:map, []}}
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:required, {:atom, :foo}, :integer}]},
                {:map, [{:required, {:atom, :foo}, :integer}]}
              ) == {:ok, {:map, [{:required, {:atom, :foo}, :integer}]}}
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:required, {:atom, :foo}, :integer}]},
                {:map, [{:optional, {:atom, :foo}, :integer}]}
              ) == {:ok, {:map, [{:required, {:atom, :foo}, :integer}]}}
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:optional, {:atom, :foo}, :integer}]},
                {:map, [{:required, {:atom, :foo}, :integer}]}
              ) == {:ok, {:map, [{:required, {:atom, :foo}, :integer}]}}
 
-      assert unify_directed_lift(
+      assert unify_lift(
                {:map, [{:optional, {:atom, :foo}, :integer}]},
                {:map, [{:optional, {:atom, :foo}, :integer}]}
              ) == {:ok, {:map, [{:optional, {:atom, :foo}, :integer}]}}
     end
 
+    test "map with subtyped and multiple matching keys" do
+      assert {:error, _} =
+               unify_lift(
+                 {:map,
+                  [
+                    {:required, {:atom, :foo}, :integer},
+                    {:required, :atom, {:union, [:integer, :boolean]}}
+                  ]},
+                 {:map, [{:required, :atom, :integer}]}
+               )
+
+      assert unify_lift(
+               {:map,
+                [
+                  {:required, {:atom, :foo}, :integer},
+                  {:required, :atom, {:union, [:integer, :boolean]}}
+                ]},
+               {:map, [{:required, :atom, {:union, [:integer, :boolean]}}]}
+             ) ==
+               {:ok,
+                {:map,
+                 [
+                   {:required, {:atom, :foo}, :integer},
+                   {:required, :atom, {:union, [:integer, :boolean]}}
+                 ]}}
+
+      assert {:error, _} =
+               unify_lift(
+                 {:map, [{:required, :atom, :integer}]},
+                 {:map,
+                  [
+                    {:required, {:atom, :foo}, :integer},
+                    {:required, :atom, {:union, [:integer, :boolean]}}
+                  ]}
+               )
+
+      assert {:error, _} =
+               unify_lift(
+                 {:map, [{:required, :atom, {:union, [:integer, :boolean]}}]},
+                 {:map,
+                  [
+                    {:required, {:atom, :foo}, :integer},
+                    {:required, :atom, {:union, [:integer, :boolean]}}
+                  ]}
+               )
+
+      assert unify_lift(
+               {:map, [{:required, :atom, :integer}]},
+               {:map,
+                [
+                  {:optional, {:atom, :foo}, :integer},
+                  {:optional, :atom, {:union, [:integer, :boolean]}}
+                ]}
+             ) == {:ok, {:map, [{:required, :atom, :integer}]}}
+
+      assert unify_lift(
+               {:map, [{:required, :atom, {:union, [:integer, :boolean]}}]},
+               {:map,
+                [
+                  {:optional, {:atom, :foo}, :integer},
+                  {:optional, :atom, {:union, [:integer, :boolean]}}
+                ]}
+             ) == {:ok, {:map, [{:required, :atom, {:union, [:integer, :boolean]}}]}}
+
+      assert unify_lift(
+               {:map,
+                [
+                  {:optional, {:atom, :foo}, :integer},
+                  {:optional, :atom, {:union, [:integer, :boolean]}}
+                ]},
+               {:map, [{:required, :atom, :integer}]}
+             ) == {:ok, {:map, [{:required, {:atom, :foo}, :integer}]}}
+
+      # TODO: FIX ME
+      # assert unify_lift(
+      #          {:map,
+      #           [
+      #             {:optional, {:atom, :foo}, :integer},
+      #             {:optional, :atom, {:union, [:integer, :boolean]}}
+      #           ]},
+      #          {:map, [{:required, :atom, {:union, [:integer, :boolean]}}]}
+      #        ) ==
+      #          {:ok,
+      #           {:map,
+      #            [
+      #              {:required, {:atom, :foo}, :integer},
+      #              {:required, :atom, {:union, [:integer, :boolean]}}
+      #            ]}}
+
+      assert {:error, _} =
+               unify_lift(
+                 {:map,
+                  [
+                    {:optional, {:atom, :foo}, :integer},
+                    {:optional, :atom, {:union, [:integer, :boolean]}}
+                  ]},
+                 {:map, [{:optional, :atom, :integer}]}
+               )
+
+      assert unify_lift(
+               {:map,
+                [
+                  {:optional, {:atom, :foo}, :integer},
+                  {:optional, :atom, {:union, [:integer, :boolean]}}
+                ]},
+               {:map, [{:optional, :atom, {:union, [:integer, :boolean]}}]}
+             ) ==
+               {:ok,
+                {:map,
+                 [
+                   {:optional, {:atom, :foo}, :integer},
+                   {:optional, :atom, {:union, [:integer, :boolean]}}
+                 ]}}
+
+      assert unify_lift(
+               {:map, [{:optional, :atom, :integer}]},
+               {:map,
+                [
+                  {:optional, {:atom, :foo}, :integer},
+                  {:optional, :atom, {:union, [:integer, :boolean]}}
+                ]}
+             ) == {:ok, {:map, [{:optional, :atom, :integer}]}}
+
+      assert unify_lift(
+               {:map, [{:optional, :atom, {:union, [:integer, :boolean]}}]},
+               {:map,
+                [
+                  {:optional, {:atom, :foo}, :integer},
+                  {:optional, :atom, {:union, [:integer, :boolean]}}
+                ]}
+             ) == {:ok, {:map, [{:optional, :atom, {:union, [:integer, :boolean]}}]}}
+    end
+
     test "union" do
       assert unify_lift({:union, []}, {:union, []}) == {:ok, {:union, []}}
       assert unify_lift({:union, [:integer]}, {:union, [:integer]}) == {:ok, {:union, [:integer]}}
@@ -267,10 +391,10 @@ defmodule Module.Types.UnifyTest do
       assert unify_lift({:union, [:integer, :atom]}, {:union, [:atom, :integer]}) ==
                {:ok, {:union, [:integer, :atom]}}
 
-      assert unify_lift({:union, [:atom]}, {:union, [{:atom, :bar}]}) ==
-               {:ok, {:union, [{:atom, :bar}]}}
+      assert unify_lift({:union, [{:atom, :bar}]}, {:union, [:atom]}) ==
+               {:ok, {:atom, :bar}}
 
-      assert {:error, {:unable_unify, {{:union, [:integer]}, {:union, [:atom]}, _}}} =
+      assert {:error, {:unable_unify, {:integer, {:union, [:atom]}, _}}} =
                unify_lift({:union, [:integer]}, {:union, [:atom]})
     end
 
@@ -282,18 +406,18 @@ defmodule Module.Types.UnifyTest do
     end
 
     test "vars" do
-      assert {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
-      assert {{:var, 1}, var_context} = new_var({:bar, [version: 1], nil}, var_context)
+      {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
+      {{:var, 1}, var_context} = new_var({:bar, [version: 1], nil}, var_context)
 
       assert {:ok, {:var, 0}, context} = unify({:var, 0}, :integer, var_context)
-      assert Types.lift_type({:var, 0}, context) == :integer
+      assert lift_type({:var, 0}, context) == :integer
 
       assert {:ok, {:var, 0}, context} = unify(:integer, {:var, 0}, var_context)
-      assert Types.lift_type({:var, 0}, context) == :integer
+      assert lift_type({:var, 0}, context) == :integer
 
       assert {:ok, {:var, _}, context} = unify({:var, 0}, {:var, 1}, var_context)
-      assert {:var, _} = Types.lift_type({:var, 0}, context)
-      assert {:var, _} = Types.lift_type({:var, 1}, context)
+      assert {:var, _} = lift_type({:var, 0}, context)
+      assert {:var, _} = lift_type({:var, 1}, context)
 
       assert {:ok, {:var, 0}, context} = unify({:var, 0}, :integer, var_context)
       assert {:ok, {:var, 1}, context} = unify({:var, 1}, :integer, context)
@@ -317,13 +441,13 @@ defmodule Module.Types.UnifyTest do
     end
 
     test "vars inside tuples" do
-      assert {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
-      assert {{:var, 1}, var_context} = new_var({:bar, [version: 1], nil}, var_context)
+      {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
+      {{:var, 1}, var_context} = new_var({:bar, [version: 1], nil}, var_context)
 
       assert {:ok, {:tuple, 1, [{:var, 0}]}, context} =
                unify({:tuple, 1, [{:var, 0}]}, {:tuple, 1, [:integer]}, var_context)
 
-      assert Types.lift_type({:var, 0}, context) == :integer
+      assert lift_type({:var, 0}, context) == :integer
 
       assert {:ok, {:var, 0}, context} = unify({:var, 0}, :integer, var_context)
       assert {:ok, {:var, 1}, context} = unify({:var, 1}, :integer, context)
@@ -333,7 +457,7 @@ defmodule Module.Types.UnifyTest do
 
       assert {:ok, {:var, 1}, context} = unify({:var, 1}, {:tuple, 1, [{:var, 0}]}, var_context)
       assert {:ok, {:var, 0}, context} = unify({:var, 0}, :integer, context)
-      assert Types.lift_type({:var, 1}, context) == {:tuple, 1, [:integer]}
+      assert lift_type({:var, 1}, context) == {:tuple, 1, [:integer]}
 
       assert {:ok, {:var, 0}, context} = unify({:var, 0}, :integer, var_context)
       assert {:ok, {:var, 1}, context} = unify({:var, 1}, :binary, context)
@@ -342,7 +466,28 @@ defmodule Module.Types.UnifyTest do
                unify_lift({:tuple, 1, [{:var, 0}]}, {:tuple, 1, [{:var, 1}]}, context)
     end
 
-    # TODO: Vars inside unions
+    # TODO: Vars inside right unions
+
+    test "vars inside left unions" do
+      {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
+
+      assert {:ok, {:var, 0}, context} =
+               unify({:union, [{:var, 0}, :integer]}, :integer, var_context)
+
+      assert lift_type({:var, 0}, context) == :integer
+
+      assert {:ok, {:var, 0}, context} =
+               unify({:union, [{:var, 0}, :integer]}, {:union, [:integer, :atom]}, var_context)
+
+      assert lift_type({:var, 0}, context) == {:union, [:integer, :atom]}
+
+      assert {:error, {:unable_unify, {:integer, {:union, [:binary, :atom]}, _}}} =
+               unify_lift(
+                 {:union, [{:var, 0}, :integer]},
+                 {:union, [:binary, :atom]},
+                 var_context
+               )
+    end
 
     test "recursive type" do
       assert {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
@@ -357,25 +502,25 @@ defmodule Module.Types.UnifyTest do
       assert {:ok, {:var, _}, context} = unify({:var, 0}, :tuple, var_context)
       assert {:ok, {:var, _}, context} = unify({:var, 1}, {:var, 0}, context)
       assert {:ok, {:var, _}, context} = unify({:var, 0}, {:var, 1}, context)
-      assert context.types[0] == {:var, 1}
-      assert context.types[1] == :tuple
+      assert context.types[0] == :tuple
+      assert context.types[1] == {:var, 0}
 
       assert {:ok, {:var, _}, context} = unify({:var, 0}, {:var, 1}, var_context)
       assert {:ok, {:var, _}, context} = unify({:var, 1}, {:var, 2}, context)
       assert {:ok, {:var, _}, _context} = unify({:var, 2}, {:var, 0}, context)
-      assert context.types[0] == :unbound
-      assert context.types[1] == {:var, 0}
-      assert context.types[2] == {:var, 1}
+      assert context.types[0] == {:var, 1}
+      assert context.types[1] == {:var, 2}
+      assert context.types[2] == :unbound
 
       assert {:ok, {:var, _}, context} = unify({:var, 0}, {:var, 1}, var_context)
 
-      assert {:error, {:unable_unify, {{:var, 0}, {:tuple, 1, [{:var, 0}]}, _}}} =
+      assert {:error, {:unable_unify, {{:var, 1}, {:tuple, 1, [{:var, 0}]}, _}}} =
                unify_lift({:var, 1}, {:tuple, 1, [{:var, 0}]}, context)
 
       assert {:ok, {:var, _}, context} = unify({:var, 0}, {:var, 1}, var_context)
       assert {:ok, {:var, _}, context} = unify({:var, 1}, {:var, 2}, context)
 
-      assert {:error, {:unable_unify, {{:var, 0}, {:tuple, 1, [{:var, 0}]}, _}}} =
+      assert {:error, {:unable_unify, {{:var, 2}, {:tuple, 1, [{:var, 0}]}, _}}} =
                unify_lift({:var, 2}, {:tuple, 1, [{:var, 0}]}, context)
     end
 
@@ -441,6 +586,12 @@ defmodule Module.Types.UnifyTest do
       refute subtype?({:tuple, 1, [:atom]}, {:tuple, 1, [{:atom, :foo}]}, new_context())
       refute subtype?({:tuple, 1, [:atom]}, {:tuple, 2, [:atom, :atom]}, new_context())
       refute subtype?({:tuple, 2, [:atom, :atom]}, {:tuple, 1, [:atom]}, new_context())
+
+      refute subtype?(
+               {:tuple, 2, [{:atom, :a}, :integer]},
+               {:tuple, 2, [{:atom, :b}, :integer]},
+               new_context()
+             )
     end
 
     test "with maps" do
@@ -508,47 +659,112 @@ defmodule Module.Types.UnifyTest do
     assert to_union([{:atom, :foo}, :binary, :atom], new_context()) ==
              {:union, [:binary, :atom]}
 
-    assert {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
+    {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
     assert to_union([{:var, 0}], var_context) == {:var, 0}
 
     assert to_union([{:tuple, 1, [:integer]}, {:tuple, 1, [:integer]}], new_context()) ==
              {:tuple, 1, [:integer]}
   end
 
+  test "flatten_union/1" do
+    context = new_context()
+    assert flatten_union(:binary, context) == [:binary]
+    assert flatten_union({:atom, :foo}, context) == [{:atom, :foo}]
+    assert flatten_union({:union, [:binary, {:atom, :foo}]}, context) == [:binary, {:atom, :foo}]
+
+    assert flatten_union({:union, [{:union, [:integer, :binary]}, {:atom, :foo}]}, context) == [
+             :integer,
+             :binary,
+             {:atom, :foo}
+           ]
+
+    assert flatten_union({:tuple, 2, [:binary, {:atom, :foo}]}, context) ==
+             [{:tuple, 2, [:binary, {:atom, :foo}]}]
+
+    assert flatten_union({:tuple, 1, [{:union, [:binary, :integer]}]}, context) ==
+             [{:tuple, 1, [:binary]}, {:tuple, 1, [:integer]}]
+
+    assert flatten_union(
+             {:tuple, 2, [{:union, [:binary, :integer]}, {:union, [:binary, :integer]}]},
+             context
+           ) ==
+             [
+               {:tuple, 2, [:binary, :binary]},
+               {:tuple, 2, [:binary, :integer]},
+               {:tuple, 2, [:integer, :binary]},
+               {:tuple, 2, [:integer, :integer]}
+             ]
+
+    {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
+    assert flatten_union({:var, 0}, var_context) == [{:var, 0}]
+
+    {:ok, {:var, 0}, var_context} = unify({:var, 0}, :integer, var_context)
+    assert flatten_union({:var, 0}, var_context) == [:integer]
+
+    {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
+    {:ok, {:var, 0}, var_context} = unify({:var, 0}, {:union, [:integer, :float]}, var_context)
+    assert flatten_union({:var, 0}, var_context) == [:integer, :float]
+
+    {{:var, 0}, var_context} = new_var({:foo, [version: 0], nil}, new_context())
+    {{:var, 1}, var_context} = new_var({:bar, [version: 1], nil}, var_context)
+    {:ok, {:var, 0}, var_context} = unify({:var, 0}, {:var, 1}, var_context)
+    assert flatten_union({:var, 0}, var_context) == [{:var, 1}]
+    assert flatten_union({:var, 1}, var_context) == [{:var, 1}]
+  end
+
   test "format_type/1" do
-    assert format_type(:binary, false) == "binary()"
-    assert format_type({:atom, true}, false) == "true"
-    assert format_type({:atom, :atom}, false) == ":atom"
-    assert format_type({:list, :binary}, false) == "[binary()]"
-    assert format_type({:tuple, 0, []}, false) == "{}"
-    assert format_type({:tuple, 1, [:integer]}, false) == "{integer()}"
+    assert format_type_string(:binary, false) == "binary()"
+    assert format_type_string({:atom, true}, false) == "true"
+    assert format_type_string({:atom, :atom}, false) == ":atom"
+    assert format_type_string({:list, :binary}, false) == "[binary()]"
+    assert format_type_string({:tuple, 0, []}, false) == "{}"
+    assert format_type_string({:tuple, 1, [:integer]}, false) == "{integer()}"
 
-    assert format_type({:map, []}, true) == "map()"
-    assert format_type({:map, [{:required, {:atom, :foo}, :atom}]}, true) == "map()"
+    assert format_type_string({:map, []}, true) == "map()"
+    assert format_type_string({:map, [{:required, {:atom, :foo}, :atom}]}, true) == "map()"
 
-    assert format_type({:map, []}, false) ==
+    assert format_type_string({:map, []}, false) ==
              "%{}"
 
-    assert format_type({:map, [{:required, {:atom, :foo}, :atom}]}, false) ==
+    assert format_type_string({:map, [{:required, {:atom, :foo}, :atom}]}, false) ==
              "%{foo: atom()}"
 
-    assert format_type({:map, [{:required, :integer, :atom}]}, false) ==
+    assert format_type_string({:map, [{:required, :integer, :atom}]}, false) ==
              "%{integer() => atom()}"
 
-    assert format_type({:map, [{:optional, :integer, :atom}]}, false) ==
+    assert format_type_string({:map, [{:optional, :integer, :atom}]}, false) ==
              "%{optional(integer()) => atom()}"
 
-    assert format_type({:map, [{:optional, {:atom, :foo}, :atom}]}, false) ==
+    assert format_type_string({:map, [{:optional, {:atom, :foo}, :atom}]}, false) ==
              "%{optional(:foo) => atom()}"
 
-    assert format_type({:map, [{:required, {:atom, :__struct__}, {:atom, Struct}}]}, false) ==
+    assert format_type_string({:map, [{:required, {:atom, :__struct__}, {:atom, Struct}}]}, false) ==
              "%Struct{}"
 
-    assert format_type(
+    assert format_type_string(
              {:map,
               [{:required, {:atom, :__struct__}, {:atom, Struct}}, {:required, :integer, :atom}]},
              false
            ) ==
              "%Struct{integer() => atom()}"
+
+    assert format_type_string({:fun, [{[], :dynamic}]}, false) == "(-> dynamic())"
+
+    assert format_type_string({:fun, [{[:integer], :dynamic}]}, false) ==
+             "(integer() -> dynamic())"
+
+    assert format_type_string({:fun, [{[:integer, :float], :dynamic}]}, false) ==
+             "(integer(), float() -> dynamic())"
+
+    assert format_type_string({:fun, [{[:integer], :dynamic}, {[:integer], :dynamic}]}, false) ==
+             "(integer() -> dynamic(); integer() -> dynamic())"
+  end
+
+  test "walk/3" do
+    assert walk(:dynamic, :acc, fn :dynamic, :acc -> {:integer, :bar} end) == {:integer, :bar}
+
+    assert walk({:list, {:tuple, [:integer, :binary]}}, 1, fn type, counter ->
+             {type, counter + 1}
+           end) == {{:list, {:tuple, [:integer, :binary]}}, 3}
   end
 end
