@@ -96,9 +96,9 @@ defmodule ExUnit.DocTest do
   values are treated as comments in Elixir code due to the leading
   `#` sign, they require special care when being used in doctests.
 
-  Imagine you have a map that contains a MapSet and is printed as:
+  Imagine you have a map that contains a `DateTime` and is printed as:
 
-      %{users: #MapSet<[:foo, :bar]>}
+      %{datetime: #DateTime<2023-06-26 09:30:00+09:00 JST Asia/Tokyo>}
 
   If you try to match on such an expression, `doctest` will fail to compile.
   There are two ways to resolve this.
@@ -106,20 +106,20 @@ defmodule ExUnit.DocTest do
   The first is to rely on the fact that doctest can compare internal
   structures as long as they are at the root. So one could write:
 
-      iex> map = %{users: Enum.into([:foo, :bar], MapSet.new())}
-      iex> map.users
-      #MapSet<[:foo, :bar]>
+      iex> map = %{datetime: DateTime.from_naive!(~N[2023-06-26T09:30:00], "Asia/Tokyo")}
+      iex> map.datetime
+      #DateTime<2023-06-26 09:30:00+09:00 JST Asia/Tokyo>
 
   Whenever a doctest starts with "#Name<", `doctest` will perform a string
   comparison. For example, the above test will perform the following match:
 
-      inspect(map.users) == "#MapSet<[:foo, :bar]>"
+      inspect(map.datetime) == "#DateTime<2023-06-26 09:30:00+09:00 JST Asia/Tokyo>"
 
   Alternatively, since doctest results are actually evaluated, you can have
-  the MapSet building expression as the doctest result:
+  the `DateTime` building expression as the doctest result:
 
-      iex> %{users: Enum.into([:foo, :bar], MapSet.new())}
-      %{users: Enum.into([:foo, :bar], MapSet.new())}
+      iex> %{datetime: DateTime.from_naive!(~N[2023-06-26T09:30:00], "Asia/Tokyo")}
+      %{datetime: DateTime.from_naive!(~N[2023-06-26T09:30:00], "Asia/Tokyo")}
 
   The downside of this approach is that the doctest result is not really
   what users would see in the terminal.
@@ -152,6 +152,12 @@ defmodule ExUnit.DocTest do
   @opaque_type_regex ~r/#[\w\.]+</
 
   defmodule Error do
+    @moduledoc """
+    Exception raised when there's an error with the syntax or semantics of a doctest.
+    """
+
+    @typedoc since: "1.16.0"
+    @type t :: %__MODULE__{message: String.t()}
     defexception [:message]
 
     @impl true
@@ -382,15 +388,13 @@ defmodule ExUnit.DocTest do
     last_expr = Macro.to_string(last_expr(expr_ast))
 
     quote do
-      # `expr_ast` may introduce variables that may be
-      # used within `expected_ast` so they both need to
-      # unquoted together here
+      # `expr_ast` may introduce variables that may be used
+      # within `expected_ast` so it needs to be unquoted here.
       value = unquote(expr_ast)
-      expected_value = unquote(expected_ast)
 
       ExUnit.DocTest.__test__(
         value,
-        expected_value,
+        unquote(expected_ast),
         unquote(doctest),
         unquote(last_expr),
         unquote(expected),
@@ -612,29 +616,39 @@ defmodule ExUnit.DocTest do
   @fences ["```", "~~~"]
 
   defp adjust_indent(lines, line_no, module) do
-    adjust_indent(:text, lines, line_no, [], 0, module)
+    adjust_text(lines, line_no, [], module)
   end
 
-  defp adjust_indent(_kind, [], line_no, adjusted_lines, _indent, _module) do
+  defp adjust_text([], line_no, adjusted_lines, _module) do
     {Enum.reverse(adjusted_lines), line_no - 1}
   end
 
-  defp adjust_indent(:text, [line | rest], line_no, adjusted_lines, indent, module) do
+  defp adjust_text([line | rest], line_no, adjusted_lines, module) do
     case String.starts_with?(String.trim_leading(line), @iex_prompt) do
       true ->
-        line_indent = get_indent(line, indent)
-        adjust_indent(:prompt, [line | rest], line_no, adjusted_lines, line_indent, module)
+        {indent, _len} = :binary.match(line, "iex")
+        adjust_code(:prompt, [line | rest], line_no, adjusted_lines, indent, module)
 
       false ->
-        adjust_indent(:text, rest, line_no + 1, adjusted_lines, indent, module)
+        adjust_text(rest, line_no + 1, adjusted_lines, module)
     end
   end
 
-  defp adjust_indent(kind, [line | rest], line_no, adjusted_lines, indent, module) do
+  defp adjust_code(_kind, [], line_no, adjusted_lines, _indent, _module) do
+    {Enum.reverse(adjusted_lines), line_no - 1}
+  end
+
+  defp adjust_code(kind, [line | rest], line_no, adjusted_lines, indent, module) do
     stripped_line = strip_indent(line, indent)
     trimmed_line = String.trim_leading(line)
+    done? = stripped_line == "" or String.starts_with?(stripped_line, @fences)
 
-    if kind != :code and trimmed_line != stripped_line do
+    overrun? =
+      if kind == :code,
+        do: not done? and byte_size(trimmed_line) > byte_size(stripped_line),
+        else: byte_size(trimmed_line) != byte_size(stripped_line)
+
+    if overrun? do
       n_spaces = if indent == 1, do: "#{indent} space", else: "#{indent} spaces"
 
       raise Error,
@@ -644,7 +658,7 @@ defmodule ExUnit.DocTest do
         indentation level mismatch on doctest line: #{inspect(line)}
 
         If you are planning to assert on the result of an `iex>` expression, \
-        make sure the result is indented at the beginning of `iex>`, which \
+        make sure the result is indented at the same level as `iex>`, which \
         in this case is exactly #{n_spaces}.
 
         If instead you have an `iex>` expression that spans over multiple lines, \
@@ -653,25 +667,18 @@ defmodule ExUnit.DocTest do
     end
 
     cond do
-      stripped_line == "" or String.starts_with?(stripped_line, @fences) ->
+      done? ->
         adjusted_lines = [{"", line_no} | adjusted_lines]
-        adjust_indent(:text, rest, line_no + 1, adjusted_lines, 0, module)
+        adjust_text(rest, line_no + 1, adjusted_lines, module)
 
       kind == :prompt or String.starts_with?(trimmed_line, @iex_prompt) or
           (kind == :maybe_prompt and String.starts_with?(trimmed_line, @dot_prompt)) ->
         line = {adjust_prompt(stripped_line, line_no, module), line_no}
-        adjust_indent(:maybe_prompt, rest, line_no + 1, [line | adjusted_lines], indent, module)
+        adjust_code(:maybe_prompt, rest, line_no + 1, [line | adjusted_lines], indent, module)
 
       true ->
         adjusted_lines = [{stripped_line, line_no} | adjusted_lines]
-        adjust_indent(:code, rest, line_no + 1, adjusted_lines, indent, module)
-    end
-  end
-
-  defp get_indent(line, current_indent) do
-    case :binary.match(line, "iex") do
-      {pos, _len} -> pos
-      :nomatch -> current_indent
+        adjust_code(:code, rest, line_no + 1, adjusted_lines, indent, module)
     end
   end
 
