@@ -64,6 +64,13 @@ defmodule ExUnit.DocTestTest.GoodModule do
   bar
   """
   def multiline_exception_test, do: :ok
+
+  @doc """
+  iex> num = 1
+  iex> ExUnit.DocTestTest.GoodModule.variable_in_expectation(num)
+  num + 1
+  """
+  def variable_in_expectation(num), do: num + 1
 end
 |> ExUnit.BeamHelpers.write_beam()
 
@@ -194,14 +201,6 @@ defmodule ExUnit.DocTestTest.Invalid do
   """
   defmacro b(), do: :ok
 
-  @doc """
-    ```
-    iex> 1 + 2
-    3
-  ```
-  """
-  def indented_not_enough, do: :ok
-
   @doc ~S'''
   ```
   iex> 1 + 2
@@ -305,6 +304,28 @@ defmodule ExUnit.DocTestTest.IndentationNotEnough do
 end
 |> ExUnit.BeamHelpers.write_beam()
 
+defmodule ExUnit.DocTestTest.FencedNotEnough do
+  @doc ~S'''
+    ```
+    iex> 1 + 2
+    3
+  ```
+  '''
+  def test_fun, do: :ok
+end
+|> ExUnit.BeamHelpers.write_beam()
+
+defmodule ExUnit.DocTestTest.IndentationNotEnoughMultilineCode do
+  @doc ~S'''
+      iex> "hello
+      ...> world"
+      "hello
+  world"
+  '''
+  def test_fun, do: :ok
+end
+|> ExUnit.BeamHelpers.write_beam()
+
 defmodule ExUnit.DocTestTest.FencedHeredocs do
   @doc ~S'''
   Receives a test and formats its failure.
@@ -443,16 +464,6 @@ defmodule ExUnit.DocTestTest.Haiku do
 end
 |> ExUnit.BeamHelpers.write_beam()
 
-defmodule ExUnit.DocTestTest.VariableInExpectation do
-  @doc """
-  iex> num = 1
-  iex> ExUnit.DocTestTest.VariableInExpectation.inc(num)
-  num + 1
-  """
-  def inc(num), do: num + 1
-end
-|> ExUnit.BeamHelpers.write_beam()
-
 defmodule ExUnit.DocTestTest.PatternMatching do
   def starting_line, do: __ENV__.line + 2
 
@@ -516,7 +527,6 @@ defmodule ExUnit.DocTestTest do
   doctest ExUnit.DocTestTest.IndentationHeredocs
   doctest ExUnit.DocTestTest.FencedHeredocs
   doctest ExUnit.DocTestTest.Haiku
-  doctest ExUnit.DocTestTest.VariableInExpectation
 
   import ExUnit.CaptureIO
 
@@ -567,14 +577,18 @@ defmodule ExUnit.DocTestTest do
 
     ExUnit.configure(seed: 0, colors: [enabled: false])
     output = capture_io(fn -> ExUnit.run() end)
+    output = String.replace(output, [IO.ANSI.red(), IO.ANSI.reset()], "")
 
     assert output =~ """
              1) doctest module ExUnit.DocTestTest.Failure (1) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line)} |
-                 #{starting_line} |  1 + * 1
-                 #{line_placeholder(starting_line)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on test/ex_unit/doc_test_test.exs:#{starting_line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{starting_line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{starting_line}:6
                 doctest:
                   iex> 1 + * 1
                   1
@@ -685,14 +699,18 @@ defmodule ExUnit.DocTestTest do
 
     ExUnit.configure(seed: 0, colors: [enabled: false])
     output = capture_io(fn -> ExUnit.run() end)
+    output = String.replace(output, [IO.ANSI.red(), IO.ANSI.reset()], "")
 
     assert output =~ """
              1) doctest ExUnit.DocTestTest.Invalid.a/0 (1) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line)} |
-                 #{starting_line} |  1 + * 1
-                 #{line_placeholder(starting_line)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on test/ex_unit/doc_test_test.exs:#{starting_line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{starting_line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{starting_line}:6
                 doctest:
                   iex> 1 + * 1
                   1
@@ -700,146 +718,174 @@ defmodule ExUnit.DocTestTest do
                   test/ex_unit/doc_test_test.exs:#{starting_line}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 6
+
     assert output =~ """
              2) doctest ExUnit.DocTestTest.Invalid.b/0 (2) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 6}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line + 6)} |
-                 #{starting_line + 6} |  1 + * 1
-                 #{line_placeholder(starting_line + 6)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on test/ex_unit/doc_test_test.exs:#{line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:6
                 doctest:
                   iex> 1 + * 1
                   1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 6}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line}: ExUnit.DocTestTest.Invalid (module)
            """
 
-    assert output =~ """
-             3) doctest ExUnit.DocTestTest.Invalid.indented_not_enough/0 (3) (ExUnit.DocTestTest.InvalidCompiled)
-                test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 15}:1: unexpected token: "`" (column 1, code point U+0060)
-                 #{line_placeholder(starting_line + 15)} |
-                 #{starting_line + 15} | `
-                 #{line_placeholder(starting_line + 15)} | ^
-                doctest:
-                  iex> 1 + 2
-                  3
-                  `
-                stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 14}: ExUnit.DocTestTest.Invalid (module)
-           """
+    line = starting_line + 15
 
     assert output =~ """
-             4) doctest ExUnit.DocTestTest.Invalid.indented_too_much/0 (4) (ExUnit.DocTestTest.InvalidCompiled)
+             3) doctest ExUnit.DocTestTest.Invalid.indented_too_much/0 (3) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 23}:3: unexpected token: "`" (column 3, code point U+0060)
-                 #{line_placeholder(starting_line + 23)} |
-                 #{starting_line + 23} |   ```
-                 #{line_placeholder(starting_line + 23)} |   ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on test/ex_unit/doc_test_test.exs:#{line}:3:
+                     error: unexpected token: "`" (column 3, code point U+0060)
+                     │
+                 #{line} │   ```
+                     │   ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:3
                 doctest:
                   iex> 1 + 2
                   3
                     ```
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 22}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line - 1}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 23
+
     assert output =~ """
-             5) doctest ExUnit.DocTestTest.Invalid.dedented_past_fence/0 (5) (ExUnit.DocTestTest.InvalidCompiled)
+             4) doctest ExUnit.DocTestTest.Invalid.dedented_past_fence/0 (4) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 31}:5: unexpected token: "`" (column 5, code point U+0060)
-                 #{line_placeholder(starting_line + 31)} |
-                 #{starting_line + 31} |     ```
-                 #{line_placeholder(starting_line + 31)} |     ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on test/ex_unit/doc_test_test.exs:#{line}:5:
+                     error: unexpected token: "`" (column 5, code point U+0060)
+                     │
+                 #{line} │     ```
+                     │     ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:5
                 doctest:
                   iex> 1 + 2
                   3
                       ```
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 30}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line - 1}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 28
+
     assert output =~ """
-             6) doctest ExUnit.DocTestTest.Invalid.invalid_utf8/0 (6) (ExUnit.DocTestTest.InvalidCompiled)
+             5) doctest ExUnit.DocTestTest.Invalid.invalid_utf8/0 (5) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 Doctest did not compile, got: (UnicodeConversionError) invalid encoding starting at <<255, 34, 41>>
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 36}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 35
+
     assert output =~ """
-             7) doctest ExUnit.DocTestTest.Invalid.misplaced_opaque_type/0 (7) (ExUnit.DocTestTest.InvalidCompiled)
+           6) doctest ExUnit.DocTestTest.Invalid.misplaced_opaque_type/0 (6) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (TokenMissingError) test/ex_unit/doc_test_test.exs:#{starting_line + 43}:7: missing terminator: } (for "{" starting at line #{starting_line + 43})
-                 #{line_placeholder(starting_line + 43)} |
-                 #{starting_line + 43} | {:ok, #Inspect<[]>}
-                 #{line_placeholder(starting_line + 43)} |       ^
+                Doctest did not compile, got: (TokenMissingError) token missing on test/ex_unit/doc_test_test.exs:#{line}:20:
+                     error: missing terminator: }
+                     │
+                 228 │ {:ok, #Inspect<[]>}
+                     │ │                  └ missing closing delimiter (expected "}")
+                     │ └ unclosed delimiter
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:20
                 If you are planning to assert on the result of an iex> expression which contains a value inspected as #Name<...>, please make sure the inspected value is placed at the beginning of the expression; otherwise Elixir will treat it as a comment due to the leading sign #.
                 doctest:
                   iex> {:ok, :oops}
                   {:ok, #Inspect<[]>}
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 43}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 40
+
     assert output =~ """
-             8) doctest ExUnit.DocTestTest.Invalid.t/0 (8) (ExUnit.DocTestTest.InvalidCompiled)
+             7) doctest ExUnit.DocTestTest.Invalid.t/0 (7) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 48}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line + 48)} |
-                 #{starting_line + 48} |  1 + * 1
-                 #{line_placeholder(starting_line + 48)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on test/ex_unit/doc_test_test.exs:#{line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:6
                 doctest:
                   iex> 1 + * 1
                   1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 48}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 48
+
     assert output =~ """
-             9) doctest ExUnit.DocTestTest.Invalid.result/0 (9) (ExUnit.DocTestTest.InvalidCompiled)
+             8) doctest ExUnit.DocTestTest.Invalid.result/0 (8) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 56}:5: syntax error before: '*'
-                 #{line_placeholder(starting_line + 56)} |
-                 #{starting_line + 56} | 1 + * 1
-                 #{line_placeholder(starting_line + 56)} |     ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on test/ex_unit/doc_test_test.exs:#{line}:5:
+                     error: syntax error before: '*'
+                     │
+                 #{line} │ 1 + * 1
+                     │     ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:5
                 doctest:
                   iex> :foo
                   iex> :bar
                   1 + * 1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 56}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 53
+
     assert output =~ """
-            10) doctest ExUnit.DocTestTest.Invalid.mixed/0 (10) (ExUnit.DocTestTest.InvalidCompiled)
+             9) doctest ExUnit.DocTestTest.Invalid.mixed/0 (9) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (TokenMissingError) test/ex_unit/doc_test_test.exs:#{starting_line + 61}:6: syntax error: expression is incomplete
-                 #{line_placeholder(starting_line + 61)} |
-                 #{starting_line + 61} |  123 +
-                 #{line_placeholder(starting_line + 61)} |      ^
+                Doctest did not compile, got: (TokenMissingError) token missing on test/ex_unit/doc_test_test.exs:#{line}:6:
+                     error: syntax error: expression is incomplete
+                     │
+                 #{line} │  123 +
+                     │      ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:6
                 doctest:
                   iex> 123 +
                   :mixed
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 61}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line}: ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 61
+
     assert output =~ """
-            11) doctest ExUnit.DocTestTest.Invalid.invalid_second/0 (11) (ExUnit.DocTestTest.InvalidCompiled)
+            10) doctest ExUnit.DocTestTest.Invalid.invalid_second/0 (10) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (TokenMissingError) test/ex_unit/doc_test_test.exs:#{starting_line + 69}:6: syntax error: expression is incomplete
-                 #{line_placeholder(starting_line + 69)} |
-                 #{starting_line + 69} |  123 +
-                 #{line_placeholder(starting_line + 69)} |      ^
+                Doctest did not compile, got: (TokenMissingError) token missing on test/ex_unit/doc_test_test.exs:#{line}:6:
+                     error: syntax error: expression is incomplete
+                     │
+                 #{line} │  123 +
+                     │      ^
+                     │
+                     └─ test/ex_unit/doc_test_test.exs:#{line}:6
                 doctest:
                   iex> 123 +
                   :mixed
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 69}: ExUnit.DocTestTest.Invalid (module)
+                  test/ex_unit/doc_test_test.exs:#{line}: ExUnit.DocTestTest.Invalid (module)
            """
 
-    assert output =~ "11 doctests, 11 failures"
+    assert output =~ "10 doctests, 10 failures"
   end
 
   test "pattern matching assertions in doctests" do
@@ -984,22 +1030,22 @@ defmodule ExUnit.DocTestTest do
     assert %{
              test_type: :doctest,
              doctest: NoImport,
-             doctest_line: 126,
-             doctest_data: %{end_line: 127}
+             doctest_line: 133,
+             doctest_data: %{end_line: 134}
            } = test1.tags
 
     assert %{
              test_type: :doctest,
              doctest: NoImport,
-             doctest_line: 129,
-             doctest_data: %{end_line: 130}
+             doctest_line: 136,
+             doctest_data: %{end_line: 137}
            } = test2.tags
 
     assert %{
              test_type: :doctest,
              doctest: NoTrailing,
-             doctest_line: 138,
-             doctest_data: %{end_line: 138}
+             doctest_line: 145,
+             doctest_data: %{end_line: 145}
            } = test3.tags
   end
 
@@ -1087,6 +1133,26 @@ defmodule ExUnit.DocTestTest do
           doctest ExUnit.DocTestTest.IndentationNotEnough
         end
       end
+
+      message =
+        ~r[test/ex_unit/doc_test_test\.exs:\d+: indentation level mismatch on doctest line: \"```\".*is exactly 2 spaces]s
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.FencedNotEnough
+        end
+      end
+
+      message =
+        ~r[test/ex_unit/doc_test_test\.exs:\d+: indentation level mismatch on doctest line: \"world\\\"\".*is exactly 4 spaces]s
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.IndentationNotEnoughMultilineCode
+        end
+      end
     end
 
     test "fails on invalid use" do
@@ -1145,13 +1211,4 @@ defmodule ExUnit.DocTestTest do
     [head | _] = Direct.__ex_unit__().tests
     assert apply(Direct, head.name, [%{}]) == {:ok, 2}
   end
-
-  defp line_placeholder(line_number) do
-    digits =
-      line_number
-      |> Integer.digits()
-      |> length()
-
-    String.duplicate(" ", digits)
-  end
 end
