@@ -253,9 +253,10 @@ defmodule IEx.Evaluator do
       if path do
         [path]
       else
-        Enum.map([".", System.get_env("IEX_HOME", "~")], fn dir ->
-          dir |> Path.join(".iex.exs") |> Path.expand()
-        end)
+        # Do not assume there is a $HOME
+        for dir <- [".", System.get_env("IEX_HOME") || System.user_home()],
+            dir != nil,
+            do: dir |> Path.join(".iex.exs") |> Path.expand()
       end
 
     path = Enum.find(candidates, &File.regular?/1)
@@ -400,7 +401,13 @@ defmodule IEx.Evaluator do
           [IEx.color(:eval_error, banner), pad(blame)]
 
         _ ->
-          [IEx.color(:eval_error, Exception.format_banner(kind, blamed, stacktrace))]
+          banner = Exception.format_banner(kind, blamed, stacktrace)
+
+          if String.contains?(banner, IO.ANSI.reset()) do
+            [banner]
+          else
+            [IEx.color(:eval_error, banner)]
+          end
       end
 
     stackdata = Exception.format_stacktrace(prune_stacktrace(stacktrace))
