@@ -79,9 +79,11 @@ defmodule IEx.HelpersTest do
     end
 
     test "errors when setting up a breakpoint with invalid guard" do
-      assert_raise CompileError, ~r"cannot find or invoke local is_whatever/1", fn ->
-        break!(URI.decode_query(_, map) when is_whatever(map))
-      end
+      assert capture_io(:stderr, fn ->
+               assert_raise CompileError, fn ->
+                 break!(URI.decode_query(_, map) when is_whatever(map))
+               end
+             end) =~ "cannot find or invoke local is_whatever/1"
     end
 
     test "errors when setting up a break with no beam" do
@@ -142,7 +144,7 @@ defmodule IEx.HelpersTest do
   describe "open" do
     @iex_helpers "iex/lib/iex/helpers.ex"
     @elixir_erl "elixir/src/elixir.erl"
-    @lists_erl "#{:code.lib_dir(:stdlib, :src)}/lists.erl"
+    @lists_erl Application.app_dir(:stdlib, "src/lists.erl")
     @httpc_erl "src/http_client/httpc.erl"
     @editor System.get_env("ELIXIR_EDITOR")
 
@@ -286,7 +288,7 @@ defmodule IEx.HelpersTest do
     end
 
     test "errors when given {file, line} is not available" do
-      assert capture_iex("open({~s[foo], 3})") ==
+      assert capture_iex("open({~s[foo], 3})") =~
                "Could not open: \"foo\". File is not available."
     end
 
@@ -315,6 +317,11 @@ defmodule IEx.HelpersTest do
   describe "runtime_info" do
     test "shows VM information" do
       assert "\n## System and architecture" <> _ = capture_io(fn -> runtime_info() end)
+
+      assert "\n## Loaded OTP applications" <> _ =
+               capture_io(fn -> runtime_info([:applications]) end)
+
+      assert "\n## Memory allocators" <> _ = capture_io(fn -> runtime_info([:allocators]) end)
     end
   end
 
@@ -325,9 +332,20 @@ defmodule IEx.HelpersTest do
       assert help =~ "Welcome to Interactive Elixir"
     end
 
-    test "prints non-Elixir module specs" do
-      assert capture_io(fn -> h(:timer.sleep() / 1) end) =~
-               "@spec sleep(time) :: :ok when time: timeout()"
+    test "prints Erlang module documentation" do
+      captured = capture_io(fn -> h(:timer) end)
+      assert captured =~ "This module provides useful functions related to time."
+    end
+
+    test "prints Erlang module function specs" do
+      captured = capture_io(fn -> h(:timer.sleep() / 1) end)
+      assert captured =~ ":timer.sleep/1"
+      assert captured =~ "-spec sleep(Time) -> ok when Time :: timeout()."
+    end
+
+    test "handles non-existing Erlang module function" do
+      captured = capture_io(fn -> h(:timer.baz() / 1) end)
+      assert captured =~ "No documentation for :timer.baz was found"
     end
 
     test "prints module documentation" do
@@ -382,6 +400,10 @@ defmodule IEx.HelpersTest do
       assert capture_io(fn -> h(def) end) =~ def_h
     end
 
+    test "prints sigil documentation" do
+      assert capture_io(fn -> h(~w//) end) =~ "Handles the sigil `~w` for list of words"
+    end
+
     test "prints __info__ documentation" do
       h_output_module = capture_io(fn -> h(Module.__info__()) end)
       assert capture_io(fn -> h(Module.UnlikelyTo.Exist.__info__()) end) == h_output_module
@@ -497,6 +519,7 @@ defmodule IEx.HelpersTest do
       impl = """
       defmodule Impl do
         @behaviour MyBehaviour
+        @doc delegate_to: {Foo, :bar, 3}
         def first(0), do: 0
         @doc "Docs for Impl.second/1"
         def second(0), do: 0
@@ -511,6 +534,14 @@ defmodule IEx.HelpersTest do
         assert c(files, ".") |> Enum.sort() == [Impl, MyBehaviour]
 
         assert capture_io(fn -> h(Impl.first() / 1) end) == """
+
+                                                def first(int)
+
+               delegate_to: Foo.bar/3
+
+               Impl.first/1 has no docs but is a callback for behaviour MyBehaviour. Showing
+               callback docs instead.
+
                @callback first(integer()) :: integer()
 
                Docs for MyBehaviour.first
@@ -533,12 +564,7 @@ defmodule IEx.HelpersTest do
 
                """
 
-        assert capture_io(fn -> h(Impl.first()) end) == """
-               @callback first(integer()) :: integer()
-
-               Docs for MyBehaviour.first
-
-               """
+        assert capture_io(fn -> h(Impl.first()) end) == capture_io(fn -> h(Impl.first() / 1) end)
 
         assert capture_io(fn -> h(Impl.second()) end) == """
 
@@ -696,9 +722,9 @@ defmodule IEx.HelpersTest do
     test "lists all callbacks for an Erlang module" do
       output = capture_io(fn -> b(:gen_server) end)
 
-      assert output =~ "@callback handle_cast(request :: term(), state :: term()) ::"
-      assert output =~ "@callback handle_info(info :: :timeout | term(), state :: term()) ::"
       assert output =~ "@callback init(args :: term()) ::"
+      assert output =~ "@callback handle_cast("
+      assert output =~ "@callback handle_info("
     end
 
     test "lists all macrocallbacks for a module" do
@@ -776,11 +802,11 @@ defmodule IEx.HelpersTest do
 
       assert capture_io(fn -> b(NoMix.run()) end) == "Could not load module NoMix, got: nofile\n"
 
-      assert capture_io(fn -> b(Exception.message() / 1) end) ==
+      assert capture_io(fn -> b(Exception.message() / 1) end) =~
                "@callback message(t()) :: String.t()\n\n"
 
       assert capture_io(fn -> b(:gen_server.handle_cast() / 2) end) =~
-               "@callback handle_cast(request :: term(), state :: term()) ::"
+               "@callback handle_cast("
     end
 
     test "prints callback documentation metadata" do
@@ -831,7 +857,7 @@ defmodule IEx.HelpersTest do
 
                @macrocallback optional_macrocallback(:bar) :: atom()
 
-               @optional_callbacks [optional_callback: 1, optional_macrocallback: 1]
+               @optional_callbacks optional_callback: 1, optional_macrocallback: 1
 
                """
       end)
@@ -898,13 +924,12 @@ defmodule IEx.HelpersTest do
     end
 
     test "prints private types" do
-      assert capture_io(fn -> t(Date.Range) end) =~ "@typep iso_days"
+      assert capture_io(fn -> t(Date.Range) end) =~ "@typep days"
     end
 
     test "prints type information" do
       assert "@type t() ::" <> _ = capture_io(fn -> t(Enum.t()) end)
       assert capture_io(fn -> t(Enum.t()) end) == capture_io(fn -> t(Enum.t() / 0) end)
-      assert "@type child_spec() ::" <> _ = capture_io(fn -> t(:supervisor.child_spec()) end)
       assert capture_io(fn -> t(URI.t()) end) == capture_io(fn -> t(URI.t() / 0) end)
     end
 
@@ -982,6 +1007,30 @@ defmodule IEx.HelpersTest do
     after
       cleanup_modules([TypeSample])
     end
+
+    test "prints all types in erlang module" do
+      captured = capture_io(fn -> t(:queue) end)
+      assert captured =~ "-type queue() :: queue(_)"
+      assert captured =~ "-opaque queue(Item)"
+    end
+
+    test "prints single type from erlang module" do
+      captured = capture_io(fn -> t(:erlang.iovec()) end)
+      assert captured =~ "-type iovec() :: [binary()]"
+      assert captured =~ "A list of binaries."
+
+      captured = capture_io(fn -> t(:erlang.iovec() / 0) end)
+      assert captured =~ "-type iovec() :: [binary()]"
+      assert captured =~ "A list of binaries."
+    end
+
+    test "handles non-existing types from erlang module" do
+      captured = capture_io(fn -> t(:erlang.foo()) end)
+      assert captured =~ "No type information for :erlang.foo was found or :erlang.foo is private"
+
+      captured = capture_io(fn -> t(:erlang.foo() / 1) end)
+      assert captured =~ "No type information for :erlang.foo was found or :erlang.foo is private"
+    end
   end
 
   describe "v" do
@@ -1035,18 +1084,23 @@ defmodule IEx.HelpersTest do
       exports = capture_io(fn -> exports(IEx.Autocomplete) end)
       assert exports == "expand/1      expand/2      exports/1     remsh/1       \n"
     end
+
+    test "handles long function names" do
+      exports = capture_io(fn -> exports(Calendar.UTCOnlyTimeZoneDatabase) end)
+
+      assert exports ==
+               "time_zone_period_from_utc_iso_days/2 time_zone_periods_from_wall_datetime/2 \n"
+    end
   end
 
   describe "import_file" do
     test "imports a file" do
       with_file("dot-iex", "variable = :hello\nimport IO", fn ->
-        capture_io(:stderr, fn ->
-          assert "** (CompileError) iex:1: undefined function variable/0" <> _ =
-                   capture_iex("variable")
-        end)
+        assert capture_io(:stderr, fn -> capture_iex("variable") end) =~
+                 "undefined variable \"variable\""
 
-        assert "** (CompileError) iex:1: undefined function puts/1" <> _ =
-                 capture_iex("puts \"hi\"")
+        assert capture_io(:stderr, fn -> capture_iex("puts \"hi\"") end) =~
+                 "undefined function puts/1"
 
         assert capture_iex("import_file \"dot-iex\"\nvariable\nputs \"hi\"") ==
                  "IO\n:hello\nhi\n:ok"
@@ -1058,13 +1112,11 @@ defmodule IEx.HelpersTest do
       dot_1 = "variable = :hello\nimport IO"
 
       with_file(["dot-iex", "dot-iex-1"], [dot, dot_1], fn ->
-        capture_io(:stderr, fn ->
-          assert "** (CompileError) iex:1: undefined function parent/0" <> _ =
-                   capture_iex("parent")
-        end)
+        assert capture_io(:stderr, fn -> capture_iex("parent") end) =~
+                 "undefined variable \"parent\""
 
-        assert "** (CompileError) iex:1: undefined function puts/1" <> _ =
-                 capture_iex("puts \"hi\"")
+        assert capture_io(:stderr, fn -> capture_iex("puts \"hi\"") end) =~
+                 "undefined function puts/1"
 
         assert capture_iex("import_file \"dot-iex\"\nvariable\nputs \"hi\"\nparent") ==
                  "IO\n:hello\nhi\n:ok\ntrue"
@@ -1106,8 +1158,32 @@ defmodule IEx.HelpersTest do
   describe "use_if_available" do
     test "uses a module only if available" do
       assert "nil" == capture_iex("use_if_available NoSuchModule")
-      assert "1" == capture_iex("use_if_available Bitwise; 1 &&& 1")
-      assert "1" == capture_iex("use_if_available Bitwise, only_operators: true; 1 &&& 1")
+    end
+  end
+
+  describe "iex> |> (and other binary operators)" do
+    test "passes previous result to the pipe" do
+      Enum.each([:~>>, :<<~, :~>, :<~, :<~>], fn op ->
+        assert capture_io(:stderr, fn -> capture_iex("42\n  #{op} IO.puts()") end) =~
+                 "undefined function #{op}/2"
+      end)
+
+      assert capture_iex("+42") =~ "42"
+      assert capture_iex("-42") =~ "-42"
+      assert capture_iex("42\n |> IO.inspect(label: \"foo\")") =~ "foo: 42"
+      assert capture_iex("[42]\n++ [24]\n|> IO.inspect(label: \"foo\")") =~ "foo: [42, 24]"
+      assert capture_iex("|> IO.puts()") =~ "(RuntimeError) v(-1) is out of bounds"
+    end
+
+    test "raises if previous expression was a match" do
+      assert capture_iex("x = 42\n|> IO.puts()") =~
+               "surround the whole pipeline with parentheses '|>'"
+
+      assert capture_iex("%{x: x} = %{x: 42}\n|> IO.puts()") =~
+               "surround the whole pipeline with parentheses '|>'"
+
+      assert capture_iex("%{x: x} = map = %{x: 42}\n|> IO.puts()") =~
+               "surround the whole pipeline with parentheses '|>'"
     end
   end
 
@@ -1129,7 +1205,7 @@ defmodule IEx.HelpersTest do
     end
 
     test "handles errors" do
-      ExUnit.CaptureIO.capture_io(fn ->
+      ExUnit.CaptureIO.capture_io(:stderr, fn ->
         with_file("sample.ex", "raise \"oops\"", fn ->
           assert_raise CompileError, fn -> c("sample.ex") end
         end)
@@ -1251,9 +1327,12 @@ defmodule IEx.HelpersTest do
     @tag :capture_log
     test "loads a given module on the given nodes" do
       assert nl([node()], :lists) == {:ok, [{:nonode@nohost, :error, :sticky_directory}]}
-      assert nl(:nonexistent_module) == {:error, :nofile}
-      assert nl([:nosuchnode@badhost], Enum) == {:ok, [{:nosuchnode@badhost, :badrpc, :nodedown}]}
       assert nl([node()], Enum) == {:ok, [{:nonode@nohost, :loaded, Enum}]}
+
+      assert nl(:nonexistent_module) == {:error, :nofile}
+
+      assert nl([:nosuchnode@badhost], Enum) ==
+               {:ok, [{:nosuchnode@badhost, :badrpc, :noconnection}]}
     end
   end
 
@@ -1279,7 +1358,8 @@ defmodule IEx.HelpersTest do
                  assert Sample.run() == :run
 
                  File.write!(filename, "defmodule Sample do end")
-                 assert {:reloaded, Sample, [Sample]} = r(Sample)
+                 assert {:reloaded, [Sample]} = r(Sample)
+                 assert {:reloaded, [Sample]} = r([Sample])
 
                  message = "function Sample.run/0 is undefined or private"
 
@@ -1287,7 +1367,7 @@ defmodule IEx.HelpersTest do
                    Sample.run()
                  end
                end) =~
-                 "redefining module Sample (current version loaded from ./Elixir.Sample.beam)"
+                 "redefining module Sample (current version loaded from Elixir.Sample.beam)"
       end)
     after
       # Clean up old version produced by the r helper
@@ -1306,7 +1386,8 @@ defmodule IEx.HelpersTest do
         assert :sample.hello() == :world
 
         File.write!(filename, other_erlang_module_code())
-        assert {:reloaded, :sample, [:sample]} = r(:sample)
+        assert {:reloaded, [:sample]} = r(:sample)
+        assert {:reloaded, [:sample]} = r([:sample])
         assert :sample.hello() == :bye
       end)
     after
@@ -1316,17 +1397,25 @@ defmodule IEx.HelpersTest do
 
   describe "pid/1,3" do
     test "builds a PID from string" do
-      assert inspect(pid("0.32767.3276")) == "#PID<0.32767.3276>"
-      assert inspect(pid("0.5.6")) == "#PID<0.5.6>"
+      assert inspect(pid("0.32767.0")) == "#PID<0.32767.0>"
+      assert inspect(pid("0.5.0")) == "#PID<0.5.0>"
 
       assert_raise ArgumentError, fn ->
         pid("0.6.-6")
       end
     end
 
+    test "builds a PID from atom" do
+      assert inspect(pid(:init)) == "#PID<0.0.0>"
+
+      assert_raise ArgumentError, fn ->
+        pid(:random_atom_ZM6pX6VwQx)
+      end
+    end
+
     test "builds a PID from integers" do
-      assert inspect(pid(0, 32767, 3276)) == "#PID<0.32767.3276>"
-      assert inspect(pid(0, 5, 6)) == "#PID<0.5.6>"
+      assert inspect(pid(0, 32767, 0)) == "#PID<0.32767.0>"
+      assert inspect(pid(0, 5, 0)) == "#PID<0.5.0>"
 
       assert_raise FunctionClauseError, fn ->
         pid(0, 6, -6)
@@ -1405,7 +1494,7 @@ defmodule IEx.HelpersTest do
       defstruct []
 
       defimpl IEx.Info do
-        def info(_), do: [{"A", "it's A"}, {:b, "it's :b"}, {'c', "it's 'c'"}]
+        def info(_), do: [{"A", "it's A"}, {:b, "it's :b"}, {~c"c", "it's 'c'"}]
       end
     end
 
