@@ -13,16 +13,16 @@ defmodule Logger do
 
     * Supports both message-based and structural logging.
 
+    * Integrate with Erlang's [`:logger`](`:logger`) and
+      support custom filters and handlers.
+
     * Formats and truncates messages on the client
-      to avoid clogging `Logger` backends.
+      to avoid clogging `Logger` handlers.
 
     * Alternates between sync and async modes to remain
-      performant when required but also apply backpressure
+      performant when required but also apply back-pressure
       when under stress.
 
-    * Support for custom filters and handlers as provided by
-      Erlang's `:logger`.
-
     * Allows overriding the logging level for a specific module,
       application or process.
 
@@ -65,7 +65,7 @@ defmodule Logger do
 
   For example, `:info` takes precedence over `:debug`. If your log
   level is set to `:info`, then all `:info`, `:notice` and above will
-  be passed to backends. If your log level is set to `:alert`, only
+  be passed to handlers. If your log level is set to `:alert`, only
   `:alert` and `:emergency` will be printed.
 
   ## Message
@@ -126,8 +126,8 @@ defmodule Logger do
     * `:crash_reason` - a two-element tuple with the throw/error/exit reason
       as first argument and the stacktrace as second. A throw will always be
       `{:nocatch, term}`. An error is always an `Exception` struct. All other
-      entries are exits. The console backend ignores this metadata by default
-      but it can be useful to other backends, such as the ones that report
+      entries are exits. The default formatter ignores this metadata by default
+      but it can be useful to certain handlers, such as the ones that report
       errors to third-party services
 
   There are two special metadata keys, `:module` and `:function`, which
@@ -275,8 +275,8 @@ defmodule Logger do
       Remember that if you want to purge log calls from a dependency, the
       dependency must be recompiled.
 
-  For example, to configure the `:backends` and purge all calls that happen
-  at compile time with level lower than `:info` in a `config/config.exs` file:
+  For example, to purge all calls that happen at compile time with level
+  lower than `:info` in a `config/config.exs` file:
 
       config :logger,
         compile_time_purge_matching: [
@@ -300,7 +300,7 @@ defmodule Logger do
 
     * `:level` - the logging level. Attempting to log any message
       with severity less than the configured level will simply
-      cause the message to be ignored. Keep in mind that each backend
+      cause the message to be ignored. Keep in mind that each handler
       may have its specific level, too. In addition to levels mentioned
       above it also supports 2 "meta-levels":
 
@@ -366,12 +366,38 @@ defmodule Logger do
   It sets up a log handler with overload protection and allows incoming events
   to be dispatched to multiple backends.
 
+  ### Filtering
+
+  You can add filters to Erlang's `:logger`. For example, to filter out logs
+  that contain a particular string, you could create a module:
+
+      defmodule LogFilter do
+        def filter(log_event, _opts) do
+          case log_event do
+            %{msg: msg} when is_binary(msg) ->
+              if msg =~ "password" do
+                :stop
+              else
+                :ignore
+              end
+
+            _ ->
+              :ignore
+          end
+        end
+      end
+
+  Then, when you start your application, such as in the
+  `c:Application.start/2` callback:
+
+      :logger.add_primary_filter(:word_filter, {&LogFilter.filter/2, []})
+
   ## Backends and backwards compatibility
 
   Prior to Elixir v1.15, custom logging could be achieved with Logger
   backends. The main API for writing Logger backends have been moved to
   the [`:logger_backends`](https://github.com/elixir-lang/logger_backends)
-  project. However, the backends are still part of Elixir for backwards
+  project. However, the backends API are still part of Elixir for backwards
   compatibility.
 
   Important remarks:
@@ -380,7 +406,7 @@ defmodule Logger do
       we assume that you want to disable the built-in logging. You can force
       logging by setting `config :logger, :default_handler, []`
 
-    * The `:console` backend configuration is automatically maped to the default
+    * The `:console` backend configuration is automatically mapped to the default
       handler and default formatter. Previously, you would set:
 
           config :logger, :console,
@@ -402,9 +428,12 @@ defmodule Logger do
       Backends, you can still set `backends: [Logger.Backends.Console]` and place
       the configuration under `config :logger, Logger.Backends.Console`. Although
       consider using the [`:logger_backends`](https://github.com/elixir-lang/logger_backends)
-      project in such case, as `Logger.Backends.Console` itself will be deprecated
+      project in such cases, as `Logger.Backends.Console` itself will be deprecated
       in future releases
 
+    * `Logger.Backends` only receive `:debug`, `:info`, `:warning`, and `:error`
+      messages. `:notice` maps to `:info`. `:warn` amps to `:warnings`.
+      All others map to `:error`
   """
 
   @type level ::
@@ -416,6 +445,13 @@ defmodule Logger do
   @levels [:error, :info, :debug] ++ @new_erlang_levels
   @metadata :logger_level
 
+  @doc ~S"""
+  Returns all the available levels.
+  """
+  @doc since: "1.16.0"
+  @spec levels() :: [level(), ...]
+  def levels(), do: @levels
+
   @doc ~S"""
   Returns the default formatter used by Logger.
 
@@ -431,7 +467,7 @@ defmodule Logger do
   ## Examples
 
   `Logger` will automatically load a default formatter into the default handler
-  on boot. However, you can use this function if you wish to programatically replace
+  on boot. However, you can use this function if you wish to programmatically replace
   a handler formatter. For example, inside tests, you might want to change the formatter
   settings:
 
@@ -831,12 +867,9 @@ defmodule Logger do
 
   defp update_translators(updater) do
     :elixir_config.serial(fn ->
-      translators = updater.(Application.fetch_env!(:logger, :translators))
-      Application.put_env(:logger, :translators, translators)
-
       with %{filters: filters} <- :logger.get_primary_config(),
            {{_, {fun, config}}, filters} <- List.keytake(filters, :logger_translator, 0) do
-        config = %{config | translators: translators}
+        config = update_in(config.translators, updater)
         :ok = :logger.set_primary_config(:filters, filters ++ [logger_translator: {fun, config}])
       end
     end)
@@ -910,37 +943,21 @@ defmodule Logger do
 
   defp add_elixir_domain(metadata), do: Map.put(metadata, :domain, [:elixir])
 
-  translations = %{
-    emergency: :error,
-    alert: :error,
-    critical: :error,
-    notice: :info
-  }
-
   for level <- @levels do
     report = [something: :reported, this: level]
-
-    extra =
-      if translation = translations[level] do
-        """
-
-
-        This is reported as \"#{translation}\" in Elixir's
-        logger backends for backwards compatibility reasons.
-
-        """
-      end
+    metadata = [user_id: 42, request_id: "xU32kFa"]
+    article = if level in [:info, :error, :alert, :emergency], do: "an", else: "a"
 
     @doc """
-    Logs a #{level} message.
+    Logs #{article} #{level} message.
 
-    Returns `:ok`.#{extra}
+    Returns `:ok`.
 
     ## Examples
 
     Logging a message (string or iodata):
 
-        Logger.#{level}("this is a #{level} message")
+        Logger.#{level}("this is #{article} #{level} message")
 
     Report message (maps or keywords):
 
@@ -950,6 +967,14 @@ defmodule Logger do
         # as map
         Logger.#{level}(#{inspect(Map.new(report))})
 
+    Report message with metadata (maps or keywords):
+
+        # as a keyword list
+        Logger.#{level}("this is #{article} #{level} message", #{inspect(metadata)})
+
+        # as map
+        Logger.#{level}("this is #{article} #{level} message", #{inspect(Map.new(metadata))})
+
     """
 
     # Only macros generated for the "new" Erlang levels are available since 1.11.0. Other
