@@ -18,19 +18,20 @@ defmodule Mix.Tasks.Compile.Leex do
   You can force compilation regardless of modification times by passing
   the `--force` option.
 
+  You must add `compilers: [:leex] ++ Mix.compilers()` to the `def project`
+  section of your `mix.exs` to run this compiler.
+
   ## Command line options
 
+    * `--all-warnings` (`--no-all-warnings`) - prints all warnings, including previous compilations
+      (default is true except on errors)
     * `--force` - forces compilation regardless of modification times
 
-    * `--all-warnings` - prints warnings even from files that do not need to be
-      recompiled
-
   ## Configuration
 
     * `:erlc_paths` - directories to find source files. Defaults to `["src"]`.
 
-    * `:leex_options` - compilation options that apply
-      to Leex's compiler.
+    * `:leex_options` - compilation options that apply to Leex's compiler.
 
       For a complete list of options, see `:leex.file/2`.
       Note that the `:report`, `:return_errors`, and `:return_warnings` options
@@ -48,19 +49,32 @@ defmodule Mix.Tasks.Compile.Leex do
     Mix.Compilers.Erlang.assert_valid_erlc_paths(source_paths)
     mappings = Enum.zip(source_paths, source_paths)
 
-    options = project[:leex_options] || []
+    leex_options = project[:leex_options] || []
 
-    unless is_list(options) do
-      Mix.raise(":leex_options should be a list of options, got: #{inspect(options)}")
+    unless is_list(leex_options) do
+      Mix.raise(":leex_options should be a list of options, got: #{inspect(leex_options)}")
     end
 
+    opts = [parallel: true, preload: fn -> preload(project) end] ++ opts
+
     Erlang.compile(manifest(), mappings, :xrl, :erl, opts, fn input, output ->
-      Erlang.ensure_application!(:parsetools, input)
-      options = options ++ @forced_opts ++ [scannerfile: Erlang.to_erl_file(output)]
+      options = leex_options ++ @forced_opts ++ [scannerfile: Erlang.to_erl_file(output)]
       :leex.file(Erlang.to_erl_file(input), options)
     end)
   end
 
+  defp preload(project) do
+    # TODO: Remove me in Elixir v2.0
+    unless :leex in List.wrap(project[:compilers]) do
+      IO.warn(
+        "in order to compile .xrl files, you must add \"compilers: [:leex] ++ Mix.compilers()\" to the \"def project\" section of #{project[:app]}'s mix.exs"
+      )
+    end
+
+    Mix.ensure_application!(:parsetools)
+    {:ok, _} = Application.ensure_all_started(:parsetools)
+  end
+
   @impl true
   def manifests, do: [manifest()]
   defp manifest, do: Path.join(Mix.Project.manifest_path(), @manifest)
