@@ -115,15 +115,12 @@ defmodule Mix.Tasks.Release do
   distribution. For example, if you need to do some prep work before
   running the actual system, like migrating your database, `eval` can
   be a good fit. Just keep in mind any application you may use during
-  eval has to be explicitly loaded and/or started.
+  eval has to be explicitly started.
 
   You can start an application by calling `Application.ensure_all_started/1`.
-  However, if for some reason you cannot start an application, maybe
-  because it will run other services you do not want, you must at least
-  load the application by calling `Application.load/1`. If you don't
-  load the application, any attempt at reading its environment or
-  configuration may fail. Note that if you start an application,
-  it is automatically loaded before started.
+  From Elixir v1.16, it is guaranteed the applications have been
+  at least loaded. In earlier versions, if you needed to load applications
+  but not start them, you also needed to call `Application.load/1`.
 
   Another way to run commands is with `rpc`, which will connect to the
   system currently running and instruct it to execute the given
@@ -392,10 +389,10 @@ defmodule Mix.Tasks.Release do
     * `:strip_beams` - controls if BEAM files should have their debug information,
       documentation chunks, and other non-essential metadata removed. Defaults to
       `true`. May be set to `false` to disable stripping. Also accepts
-     `[keep: ["Docs", "Dbgi"]]` to keep certain chunks that are usually stripped.
-     You can also set the `:compress` option to true to enable individual
-     compression of BEAM files, although it is typically preferred to compress
-     the whole release instead.
+      `[keep: ["Docs", "Dbgi"]]` to keep certain chunks that are usually stripped.
+      You can also set the `:compress` option to true to enable individual
+      compression of BEAM files, although it is typically preferred to compress
+      the whole release instead.
 
     * `:cookie` - a string representing the Erlang Distribution cookie. If this
       option is not set, a random cookie is written to the `releases/COOKIE` file
@@ -578,6 +575,21 @@ defmodule Mix.Tasks.Release do
   )
   ```
 
+  Inside `env.sh` and `env.bat` files you can access command-line arguments given to release commands.
+  For example, given this `env.sh.eex`:
+
+  ```bash
+  echo $@
+  ```
+
+  or this `env.bat.eex`:
+
+  ```bash
+  echo %*
+  ```
+
+  starting the release with `bin/myapp start --foo bar baz` will print `start --foo bar baz`.
+
   ## Application configuration
 
   Mix provides two mechanisms for configuring the application environment
@@ -1009,7 +1021,7 @@ defmodule Mix.Tasks.Release do
     * `--no-deps-check` - does not check dependencies
     * `--no-elixir-version-check` - does not check Elixir version
     * `--no-compile` - does not compile before assembling the release
-    * `--overwrite` - if there is an existing release version, overwrite it
+    * `--overwrite` - overwrite existing files instead of prompting the user for action
     * `--path` - the path of the release
     * `--quiet` - does not write progress to the standard output
     * `--version` - the version of the release
