@@ -19,14 +19,6 @@ defmodule Mix.Tasks.Compile.ElixirTest do
 
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
-
-      File.write!("lib/a.ex", """
-      defmodule A, do: :ok
-
-      # Also make sure that we access the ebin directory during compilation
-      true = to_charlist(Mix.Project.compile_path()) in :code.get_path()
-      """)
-
       Mix.Tasks.Compile.Elixir.run(["--verbose"])
 
       assert File.regular?("_build/shared/lib/sample/ebin/Elixir.A.beam")
@@ -40,14 +32,6 @@ defmodule Mix.Tasks.Compile.ElixirTest do
   test "compiles a project with per environment build" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
-
-      File.write!("lib/a.ex", """
-      defmodule A, do: :ok
-
-      # Also make sure that we access the ebin directory during compilation
-      true = to_charlist(Mix.Project.compile_path()) in :code.get_path()
-      """)
-
       Mix.Tasks.Compile.Elixir.run(["--verbose"])
 
       assert File.regular?("_build/dev/lib/sample/ebin/Elixir.A.beam")
@@ -788,7 +772,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
     end)
   end
 
-  test "recompiles mtime changed files if content changed but not length" do
+  test "compiles mtime changed files if content changed but not length" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
@@ -888,7 +872,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
     end)
   end
 
-  test "recompiles size changed files" do
+  test "compiles size changed files" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
       past = @old_time
@@ -910,7 +894,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
     end)
   end
 
-  test "recompiles dependent changed modules" do
+  test "compiles dependent changed modules" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
       File.write!("lib/a.ex", "defmodule A, do: B.module_info()")
@@ -930,7 +914,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
     end)
   end
 
-  test "recompiles dependent changed modules without beam files" do
+  test "compiles dependent changed modules without beam files" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
 
@@ -947,10 +931,11 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert File.regular?("_build/dev/lib/sample/ebin/Elixir.A.beam")
       assert File.regular?("_build/dev/lib/sample/ebin/Elixir.B.beam")
 
+      # Compile directly so it does not point to a .beam file
       Code.put_compiler_option(:ignore_module_conflict, true)
       Code.compile_file("lib/b.ex")
-      force_recompilation("lib/a.ex")
 
+      force_recompilation("lib/a.ex")
       Mix.Tasks.Compile.Elixir.run(["--verbose"])
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
     end)
@@ -958,7 +943,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
     Code.put_compiler_option(:ignore_module_conflict, false)
   end
 
-  test "recompiles dependent changed modules even on removal" do
+  test "compiles dependent changed modules even on removal" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
       File.write!("lib/a.ex", "defmodule A, do: B.module_info()")
@@ -979,7 +964,34 @@ defmodule Mix.Tasks.Compile.ElixirTest do
     end)
   end
 
-  test "recompiles dependent changed external resources" do
+  test "compiles dependent changed on conflict" do
+    in_fixture("no_mixfile", fn ->
+      Mix.Project.push(MixTest.Case.Sample)
+
+      assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
+      assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
+      assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
+      purge([A, B])
+
+      capture_io(:stderr, fn ->
+        File.write!("lib/a.ex", "defmodule B, do: :not_ok")
+        assert {:ok, [_ | _]} = Mix.Tasks.Compile.Elixir.run(["--verbose"])
+        assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
+        assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
+        purge([A, B])
+      end)
+
+      capture_io(:stderr, fn ->
+        File.write!("lib/a.ex", "defmodule A, do: :ok")
+        assert {:ok, []} = Mix.Tasks.Compile.Elixir.run(["--verbose"])
+        assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
+        assert_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
+        purge([A, B])
+      end)
+    end)
+  end
+
+  test "compiles dependent changed external resources" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
       tmp = tmp_path("c.eex")
@@ -987,7 +999,11 @@ defmodule Mix.Tasks.Compile.ElixirTest do
 
       File.write!("lib/a.ex", """
       defmodule A do
+        # Listing directories as external resources
+        # is not valid but let's ensure we don't crash
+        @external_resource "lib"
         @external_resource "lib/a.eex"
+
         @external_resource #{inspect(tmp)}
         def a, do: :ok
       end
@@ -1026,6 +1042,12 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
       assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
       refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
+
+      # Recompiles if directories are stale
+      File.touch!("lib", {{2038, 1, 1}, {0, 0, 0}})
+      assert Mix.Tasks.Compile.Elixir.run(["--verbose"]) == {:ok, []}
+      assert_received {:mix_shell, :info, ["Compiled lib/a.ex"]}
+      refute_received {:mix_shell, :info, ["Compiled lib/b.ex"]}
     end)
   after
     File.rm(tmp_path("c.eex"))
@@ -1421,8 +1443,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
 
         assert %Diagnostic{
                  file: ^file,
+                 source: ^file,
                  severity: :warning,
-                 position: 2,
+                 position: {2, 13},
                  compiler_name: "Elixir",
                  message: ^message
                } = diagnostic
@@ -1435,8 +1458,9 @@ defmodule Mix.Tasks.Compile.ElixirTest do
 
         assert %Diagnostic{
                  file: ^file,
+                 source: ^file,
                  severity: :warning,
-                 position: 2,
+                 position: {2, 13},
                  compiler_name: "Elixir",
                  message: ^message
                } = diagnostic
@@ -1452,17 +1476,16 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       IO.warn "warning", [{nil, nil, 0, file: 'lib/foo.txt', line: 3}]
       """)
 
-      file = Path.absname("lib/foo.txt")
-
       capture_io(:stderr, fn ->
         assert {:ok, [diagnostic]} = Mix.Tasks.Compile.Elixir.run([])
 
         assert %Diagnostic{
-                 file: ^file,
+                 file: nil,
                  severity: :warning,
-                 position: 3,
+                 position: 0,
                  compiler_name: "Elixir",
-                 message: "warning"
+                 message: "warning",
+                 stacktrace: [{nil, nil, 0, [file: ~c"lib/foo.txt", line: 3]}]
                } = diagnostic
       end)
     end)
@@ -1473,25 +1496,44 @@ defmodule Mix.Tasks.Compile.ElixirTest do
       Mix.Project.push(MixTest.Case.Sample)
       File.mkdir_p!("lib")
 
+      File.write!("lib/b.ex", """
+      defmodule B do
+        def unused_var(unused_var), do: :ok
+      end
+      """)
+
+      assert capture_io(:stderr, fn ->
+               assert {:ok, [_diagnostic]} = Mix.Tasks.Compile.Elixir.run([])
+             end) =~ "unused_var"
+
+      file = Path.absname("lib/a.ex")
+
       File.write!("lib/a.ex", """
       defmodule A do
         def my_fn(), do: $$$
       end
       """)
 
-      file = Path.absname("lib/a.ex")
+      refute capture_io(:stderr, fn ->
+               purge([B])
+               assert {:error, [_warning, error]} = Mix.Tasks.Compile.Elixir.run([])
 
-      capture_io(:stderr, fn ->
-        assert {:error, [diagnostic]} = Mix.Tasks.Compile.Elixir.run([])
+               assert %Diagnostic{
+                        file: ^file,
+                        source: ^file,
+                        severity: :error,
+                        position: {2, 20},
+                        message: "** (SyntaxError) invalid syntax found on lib/a.ex:2:" <> _,
+                        compiler_name: "Elixir"
+                      } = error
+             end) =~ "unused_var"
 
-        assert %Diagnostic{
-                 file: ^file,
-                 severity: :error,
-                 position: {2, 20},
-                 message: "** (SyntaxError) lib/a.ex:2:" <> _,
-                 compiler_name: "Elixir"
-               } = diagnostic
-      end)
+      assert capture_io(:stderr, fn ->
+               purge([B])
+
+               assert {:error, [_warning, _error]} =
+                        Mix.Tasks.Compile.Elixir.run(["--all-warnings"])
+             end) =~ "unused_var"
     end)
   end
 
@@ -1513,6 +1555,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
 
         assert %Diagnostic{
                  file: ^file,
+                 source: ^file,
                  severity: :error,
                  position: 2,
                  message: "** (KeyError) key :invalid_key not found" <> _,
@@ -1578,7 +1621,7 @@ defmodule Mix.Tasks.Compile.ElixirTest do
           if Code.ensure_loaded?(B) and not function_exported?(B, :foo, 0) do
             :ok
           else
-            IO.warn("AFTER_VERIFY", Macro.Env.stacktrace(__ENV__))
+            IO.warn("AFTER_VERIFY", __ENV__)
           end
         end
       end
